#include <u.h>
#include <libc.h>
#include <draw.h>
#include <event.h>

#include "maze.h"

char		*TheseusImage = "images/theseus.1";
char		*MinotaurImage = "images/minotaur.1";
char		*WinImage = "images/win.1";
char		*DeadImage = "images/dead.1";
char		*ExitImage = "images/exit.1";


char *buttons[] = 
{
	"restart",
	"exit",
	0
};

Menu menu = 
{
	buttons
};

Image *
eallocimage(Rectangle r, int repl, uint color)
{
	Image *tmp;

	tmp = allocimage(display, r, screen->chan, repl, color);
	if(tmp == nil)
		sysfatal("cannot allocate buffer image: %r");

	return tmp;
}

Image *
eloadfile(char *path)
{
	Image *img;
	int fd;

	fd = open(path, OREAD);
	if(fd < 0) {
		fprint(2, "cannot open image file %s: %r\n", path);
		exits("image");
	}
	img = readimage(display, fd, 0);
	if(img == nil)
		sysfatal("cannot load image: %r");
	close(fd);
	
	return img;
}

void
allocimages(void)
{
	Rectangle one = Rect(0, 0, BoardX, BoardY);
	
	img 		= eallocimage(rectsubpt(screen->r, screen->r.min), 0, DWhite);
	beven	= eallocimage(one, 0, DPaleyellow);
	bodd	= eallocimage(one, 0, DDarkyellow);
	wall		= eallocimage(one, 0, 0x888888FF);

	text 		= eallocimage(one, 0, DBluegreen);

	theseus = eloadfile(TheseusImage);
	minotaur = eloadfile(MinotaurImage);
	dead = eloadfile(DeadImage);
	win = eloadfile(WinImage);
	eximg = eloadfile(ExitImage);
}

void
endgame(int won)
{
	level.done = 1;

	if(won) {
		drawwin();
		drawscreen();
		sleep(3000);
		if(level.index == Numlevels - 1) {
			print("thank you for playing, please send extra levels!\n");
			exits(nil);
		}
		loadlevel(++level.index);
		drawlevel();
		drawscreen();
	} else {
		drawdead();
		drawscreen();
	}
}

void
eresized(int new)
{
	int fd;
	Point p = Pt(SizeX, SizeY);

	if(new && getwindow(display, Refnone) < 0) {
		sysfatal("can't reattach to window");
	}
	
	fd = open("/dev/wctl", OWRITE);
	if(fd >= 0){
		fprint(fd, "resize -dx %d -dy %d", p.x, p.y);
		close(fd);
	}

	drawlevel();
	drawscreen();
}

void 
main(int argc, char **argv)
{
	Mouse m;
	Event e;

	USED(argc);
    USED( argv);
	srand(time(0));

	if(initdraw(nil, nil, "minotaur maze") < 0)
		sysfatal("initdraw failed: %r");

	einit(Emouse|Ekeyboard);

	allocimages();

	loadlevel(0);

	eresized(0);

	for(;;) {
		switch(event(&e)) {
		case Emouse:
			m = e.mouse;
			if(m.buttons&4)
				switch(emenuhit(3, &m, &menu)) {
				case 0:
					loadlevel(level.index);
					drawlevel();
					drawscreen();
					break;
				case 1:
					exits(nil);
				}
			break;

		case Ekeyboard:
			if(level.done)
				break;
			switch(e.kbdc) {
			case 127:
				exits(nil);
			case 61454:
				moveup();
				break;
			case 0x80:
				movedown();
				break;
			case 61457:
				moveleft();
				break;
			case 61458:
				moveright();
				break;
			case ' ':
				break;
			}
			drawscreen();
			sleep(300);
			moveminotaur();

			if(eqpt(level.minotaur, level.theseus)) 
				endgame(0);
			else if(eqpt(level.exit, level.theseus))
				endgame(1);
			break;
		}
	}
}
